////////////////////////////////////////////////////////////////////////
//
//	CComPort.cpp: implementation of the CComPort class
//
//	Author: Jurgis Armanavichius
//
//	Based on original CSerialPort class by Remon Spekreijse (thanks)
//
////////////////////////////////////////////////////////////////////////

#include <string.h>
#include <windows.h>

#define COM_PORT_MAX_NAME_LENGTH  20

class CComPort
{
public:
	CComPort();
	virtual ~CComPort();

	void PresetComParameters(DWORD baud,BYTE databits,BYTE parity,BYTE stopbits);
	BOOL ChangeComParameters(DWORD baud,BYTE databits,BYTE parity,BYTE stopbits);
	BOOL OpenCom(LPCTSTR lpPortName);
	void CloseCom();
	DWORD CheckInQueue();
	BOOL GetByte(LPBYTE rByte,DWORD dwMilliseconds);
	BOOL PutByte(BYTE wByte);
	DWORD ReadBlock(LPBYTE lpBuffer,DWORD nMax);
	BOOL WriteBlock(LPBYTE lpBuffer,DWORD dwBufferLength);
	BOOL GetOpenedStatus(void);
	
protected:
	char	m_cComPortName[COM_PORT_MAX_NAME_LENGTH+1];
	HANDLE	m_hCommPort;
	BOOL	m_bOpened;
	DWORD	m_Baud;
	BYTE	m_DataBits;
	BYTE	m_Parity;
	BYTE	m_StopBits;
};

//////////////////////////////////////////////////////////////////////
// CComPort class: com_port
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CComPort::CComPort()
{
  strcpy(m_cComPortName,"");
  m_bOpened  = FALSE;
  m_Baud     = CBR_9600;
  m_DataBits = 8;
  m_Parity   = NOPARITY;
  m_StopBits = ONESTOPBIT;
}

CComPort::~CComPort()
{
  CloseCom();
}

void CComPort::PresetComParameters(DWORD baud,BYTE databits,BYTE parity,BYTE stopbits)
{
  m_Baud     = baud;
  m_DataBits = databits;
  m_Parity   = parity;
  m_StopBits = stopbits;
}

BOOL CComPort::ChangeComParameters(DWORD baud,BYTE databits,BYTE parity,BYTE stopbits)
{
  char	com_port_name[COM_PORT_MAX_NAME_LENGTH+1];

  if(m_bOpened == FALSE) return FALSE;	// You must open COM port first
  strcpy(com_port_name,m_cComPortName);
  CloseCom();
  m_Baud     = baud;
  m_DataBits = databits;
  m_Parity   = parity;
  m_StopBits = stopbits;
  return OpenCom(com_port_name);
}

BOOL CComPort::OpenCom(LPCTSTR lpPortName)
{
  if(m_bOpened) return FALSE;	// we are already opened, so OpenCom failed

//	save the COM port's name		

  if(strlen(lpPortName) > COM_PORT_MAX_NAME_LENGTH) return FALSE; // ComPort name too long!
  strcpy(m_cComPortName,lpPortName);

  BOOL fSuccess;
  DCB  dcb;
	
  m_hCommPort = CreateFile(m_cComPortName,
		GENERIC_READ | GENERIC_WRITE,
		0,		// exclusive access 
		NULL,	// no security attrs 
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL
		);
	

  if(m_hCommPort == INVALID_HANDLE_VALUE) return FALSE;	// OpenCom failed

//	Setup new connection:	

  GetCommState(m_hCommPort, &dcb);

  dcb.BaudRate = m_Baud;
  dcb.ByteSize = m_DataBits;
  dcb.Parity   = m_Parity;
  dcb.StopBits = m_StopBits;
  dcb.fOutxDsrFlow = 0;	// we don't need handshake 
  dcb.fDtrControl  = 0;
  dcb.fOutxCtsFlow = 0;
  dcb.fRtsControl  = 0;
  dcb.fBinary = TRUE ;
  dcb.fParity = TRUE ;

  fSuccess = SetCommState(m_hCommPort, &dcb);

  if(!fSuccess) return FALSE;	// Can't SetCommState -- OpenCom failed

  PurgeComm(m_hCommPort,PURGE_TXABORT | PURGE_RXABORT |
			PURGE_TXCLEAR | PURGE_RXCLEAR );	// Clean up COM port

  m_bOpened = TRUE;
  return TRUE;	// OpenCom OK
}

void CComPort::CloseCom()
{
  if(m_bOpened) {
    m_bOpened = FALSE ;
    PurgeComm(m_hCommPort,PURGE_TXABORT | PURGE_RXABORT |
			  PURGE_TXCLEAR | PURGE_RXCLEAR );	// Clean up COM port
    CloseHandle(m_hCommPort) ;
  }
}

DWORD CComPort::CheckInQueue()
{
  COMSTAT    ComStat ;
  DWORD      dwErrorFlags;
  DWORD      dwLength;

  if(!m_bOpened) return 0;	// we are not connected, return 0

  if (ClearCommError( m_hCommPort, &dwErrorFlags, &ComStat )) {
    dwLength = ComStat.cbInQue;
  }
  else {
    dwLength = 0;
  }
  return dwLength;
}

#define SLEEP_TIME		5

BOOL CComPort::GetByte(LPBYTE rByte,DWORD dwMilliseconds)
{
  BYTE  byt;
  DWORD pause_cnt;

  if (!m_bOpened) return FALSE;	// we are not connected
  pause_cnt = dwMilliseconds/SLEEP_TIME;	// we pause in SLEEP_TIME milliseconds increment
  while(1) {
    if(ReadBlock(&byt,1) > 0) {
      *rByte = byt;
      return TRUE;
    }
    if(pause_cnt == 0) break;
    Sleep(SLEEP_TIME);
    pause_cnt--;
  }
  return FALSE;
}

BOOL CComPort::PutByte(BYTE wByte)
{
  return WriteBlock(&wByte,1);
}

DWORD CComPort::ReadBlock(LPBYTE lpBuffer,DWORD nMax)
{
  BOOL       fReadStat ;
  COMSTAT    ComStat ;
  DWORD      dwErrorFlags;
  DWORD      dwLength;

  if(!m_bOpened) return (DWORD)-1;	// we are not connected

//	get number of bytes in queue

  if (ClearCommError( m_hCommPort, &dwErrorFlags, &ComStat )) {
    dwLength = min( (DWORD) nMax, ComStat.cbInQue ) ;
  }
  else {
    dwLength = 0;
  }

  if(dwLength > 0) {
    fReadStat = ReadFile(m_hCommPort,lpBuffer,dwLength,&dwLength,NULL);
    if(!fReadStat) {
      dwLength = 0;
      ClearCommError(m_hCommPort,&dwErrorFlags,&ComStat) ;
    }
  }
  return dwLength;
}

BOOL CComPort::WriteBlock(LPBYTE lpBuffer, DWORD dwBufferLength)
{
  DWORD dwBytesWritten;
  BOOL  bResult;
	
  if (!m_bOpened) return FALSE;	// we are not connected
	
  bResult = WriteFile(m_hCommPort,lpBuffer,dwBufferLength,&dwBytesWritten,NULL);

  if(!bResult) return FALSE;

  return TRUE;
}

BOOL CComPort::GetOpenedStatus(void)
{
  return m_bOpened;
}
